/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
* RC CAR DRAG RACE MANAGER                                                         Version 3  09/12/2021 *
* Created by: W. van Leeuwen (Billv923@outlook.com)                                                      *
*                                                                                                        *
* This sketch provides the display and race management interface for a 2-Lane 1/10 scale Drag Race       *
* Timer. It works in conjunction with the drag race timer software resident on the Arduino based RC car  *
* track timer unit. The Arduino timer unit provides the hardware interfaces for the start line staging   *
* sensors, 6 Ft & 66 Ft sensors, trap sensors, finish line sensors and the interface to drive the        *
* Christmas Tree lights. The timer unit's software performs track management functions, makes the track  *
* time measurements & calculations and controls the cycling of the Xmas Tree lights. Race data is sent   *
* to the PC running this sketch via the serial (USB) interface.                                          *
*                                                                                                        *
* NOTE - This version operates in conjunction with Arduino software RC_DRT_Timer_Ver3.ino                *
*                                                                                                        *
* This version provides the following features:                                                          *
*   1. Display of Racer's ID number, Reaction Time, 6 Ft Time, 66 Ft time, Trap MPH, Elapsed Time,       *
*      Overall Time and race status (Win, Draw or Disq.)                                                 *
*   2. Track status monitoring & display                                                                 *
*   3. Ability to select between Standard, Pro or Street Outlaw Xmas Tree functionality (Key F1)         *
*   4. Ability to select between manual race or auto race start (Key F2)                                 *
*   5. Ability to set timer to 1 or 2 racers (Key F3)                                                    *
*   6. Ability to command the Arduino timer to initiate the Christmas Tree lamp test (Key F4)            *
*   7. Ability to view and print results from prior ran races                                            *
*   8. Race duration counter to display number of seconds elapsed while the race is in progress          *
*   9. Automatically save race results to a comma-delimited (.csv) file                                  *
*  10. Capability to load and continue from a previously saved race event                                *
*  11. Generate and print racer's time slips                                                             *
*  12. Process timer reset & next race requests from remote hand-held pushbutton                         *
*  13. Displays a negative reaction time in red when racer red-lights (Fouls) to show how early he left  *
*      the start line.                                                                                   *
*  14. Ability to command the Arduino timer to initiate the Christmas Tree lamp test (Key F5)            *
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
   
import processing.serial.*;                         //Use the serial library
import static javax.swing.JOptionPane.*;            //Use the option pane dialog box library

/* * * * * * * * * * * * * * * * * * * * USER DEFINED PARAMETERS * * * * * * * * * * * * * * * * * * * * * * */
String Disp_Title = "Organization Name Goes Here";  //Text string to hold display title
String TS_Header1 = "DFW Winternationals";          //Time Slip header line 1 (33 Char Max)
String TS_Header2 = "RC Car Drag Racing";           //Time Slip header line 2 (33 Char Max)
String TS_Footer1 = "Thanks for racing with us";    //Time Slip footer line 1 (33 Char Max)
String TS_Footer2 = " ";                            //Time Slip footer line 2 (33 Char Max)
int Autoprintcopies = 0;                            //Number of copies to automatically print after race
//Set the file path for printing the Time Slip file
String File_Path = "C:/Users/Bill/Timers/RC_DRT_Timers/RC_DRT_Ver3/RC_DRT_RaceManager_Ver3/Timeslip.txt";
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

// Initialization
  int i, j, k, n, x, y;                     //working integer variables
  int Rx, Gx, Bx;                           //Background color RGB values
  int Yr, Mo, Day, Hr, Min;                 //Integer variables for creating date-time based filename
  int Sec;                                  //Clock seconds variable for use by the Race in Progress counter
  int Counter = 0;                          //Variable for holding race in progress duration (seconds)
  int Num_Lanes = 2;                        //Set the number of lanes
  int Race_State = 0;                       //0=Not Ready, 1=Ready, 2=Staged, 3=Countdown, 4=Racing, 5=Finished
  int Msg_Number = 0;                       //0=None, 1=Msg1, 2=Msg2, 3=Msg3, 4=Msg4, etc
  int Race_Number = 0;                      //Race number selected/displayed
  int Races_Ran = 0;                        //Number of races ran
  int Max_Races = 999;                      //Maximum number of races allowed
  int Tablerow = -1;                        //Results table row index pointer
  int lf = 10;                              //ASCII Linefeed
  int Index1;                               //Table index pointer
  int DQ1flag = 0;                          //Lane 1 disqualification flag
  int DQ2flag = 0;                          //Lane 2 disqualification flag
  int Win1flag = 0;                         //Lane 1 win flag
  int Win2flag = 0;                         //Lane 2 win flag
  int L1Color;                              //Used for holding color value for Lane 1 status
  int L2Color;                              //Used for holding color value for Lane 2 status
  int [] Place = new int[2];                //Array for storing lane finish order (2 lanes)
  boolean Canceledflag = false;             //Used to determine if file selection was canceled
  boolean PracticeRaceflag = false;         //Indicates when practice run data is available to print
  boolean NextRaceflag = true;              //Indicates when Next Race button is to be active
  boolean Racefile_Selected = false;        //Indicates when a previously saved race event file has been selected
  boolean RaceContinueflag = false;         //Indicates continuation of previously saved race event
  boolean Racer1flag = false;               //Set to true when Racer1 ID# has been entered
  boolean Racer2flag = false;               //Set to true when Racer2 ID# has been entered
  boolean Holdflag = false;                 //Set to true when viewing prior ran race results
  boolean NoComflag = false;                //Set when no COM ports are detected in setup routine
  boolean Stdtreeflag = true;               //Set when Xmas tree set to Standard sportsman mode
  boolean Protreeflag = false;              //Set when Xmas tree set to Professional mode
  boolean Outlawtreeflag = false;           //Set when Xmas tree set to street outlaw mode
  boolean Autostartflag = false;            //Set to true when timer auto start is selected
  boolean OneRacer_flag = false;            //Set to true when timer is set to a single racer
  float [] ReactTime = new float[2];        //Array for storing racer's reaction time values (2 lanes)
  float [] Trk6ftTime = new float[2];       //Array for storing racer's 6 Foot time values (2 lanes)
  float [] Trk66ftTime = new float[2];      //Array for storing racer's 66 Foot time values (2 lanes)
  float [] TrapTime = new float[2];         //Array for storing racer's Trap time values (2 lanes)
  float [] ElapsedTime = new float[2];      //Array for storing racer's elapsed time values (2 lanes)
  float [] OverallTime = new float[2];      //Array for storing racer's overall race time values (2 lanes)
  float [] Speed = new float[2];            //Array for storing racer's speed in MPH (2 lanes)
  float Value;                              //Working Floating point variable
  String Str1, Str2, Str3, Str4 = "";       //Working string variables
  String Race_Filename = "";                //Used to store file path/name of selected file
  String COMx, COMlist = "";                //COM Port check/select routine variables
  String Portname = "";                     //COM Port name variable
  String Datafile = "";                     //String variable for the save data file name
  String TS_file="Timeslip.txt";            //String variable for the Time Slip text file name
  String TS_date="";                        //String variable to hold time Slip date stamp
  String TS_time="";                        //String variable to hold time Slip time stamp
  String Time_String = "";                  //String variable for temp storage of race time values
  String XmasTree_Mode = "M0";              //M0=Standard mode, M1=Pro mode
  String Msg1 = "";                         //String variable to hold lane sensor error message
  String Msg1a = "Sensor(s)";
  String Msg1b = " not ready. Fix problem & click Timer Reset";
  String Msg2 = "Hand-held race start switch not ready. Fix problem & click Timer Reset";
  String Msg3 = "Click Next Race button to prep for next race";
  String Msg4 = "Click Timer Reset to run a practice race or Next Race button to prep for first official race";
  String Msg5 = "Remote RESET detected";
  String Msg6 = "Remote START detected";
  String Msg7 = "Must be in Ready or Staged state & next Race# selected to enter Racer ID Number";
  String Msg8 = "Must be in Ready or Finished state to enter Max Duration Code";
  String Msg9 = "Must be in Ready or Finished state to select Xmas Tree mode";
  String Msg10 = "Must be in Ready or Finished state to select Auto/Manual Start mode";
  String Msg11 = "OPTIONAL: Select Racer ID boxes above to enter or change a Racer ID/Car#";
  String Msg12 = "Must be in 2 racer mode to enter Lane 2 Racer ID/Car#";
  String prompt = "";                       //Variable used for storing popup message text
  String L1status = "";                     //Variable for holding lane 1 race status
  String L2status = "";                     //Variable for holding lane 2 race status
  String inString = "";                     //Input string from serial port
  String Prt_Cmd = "";                      //String for holding print command
  String Racer1 = " ";                      //String for holding lane 1 Racer ID Number
  String Racer1a = " ";                     //Working variable for holding lane 1 Racer ID Number
  String Racer2 = " ";                      //String for holding lane 2 Racer ID Number
  String Racer2a = " ";                     //Working variable for holding lane 2 Racer ID Number
  PFont font;
  Table Resultstable;                       //Define table named 'Resultstable'
  PrintWriter TS_output;                    //Create an output variable for writing to the Timeslip.txt file
  PImage logo;                              //Define an image variable as 'logo'
 
  Serial myPort;                            //Set the Serial port
  
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void setup() {
  size(1000, 680);                          //Set display area in pixels
  Rx = 140;                                 //Set background color red hue
  Gx = 200;                                 //Set background color green hue
  Bx = 230;                                 //Set background color blue hue
  background(Rx, Gx, Bx);                   //Set display area background color
  logo = loadImage("BillV_Logo.png");       //Load "Bill V's Derby Timers" Logo
  image(logo, 880, 620);                    //And display it in lower right corner
  Prt_Cmd = "notepad.exe /p " + File_Path;  //Set the time slip print command string
  
  //Create unique date-time based file name so that a new file is created each time the program is run.
  //This will prevent previous files from being accidentally overwritten. Note that by default the file
  //is saved to the same folder/directory that contains this sketch.
  Yr = year();                              //Get year value from PC clock
  Mo = month();                             //Get month value from PC clock
  Day = day();                              //Get day value from PC clock
  Hr = hour();                              //Get hour value from PC clock
  Min = minute();                           //Get minute value from PC clock
  Datafile = "Race_Results_"+nf(Yr,4)+nf(Mo,2)+nf(Day,2)+nf(Hr,2)+nf(Min,2)+".csv";
  
  //Initialize the 'Resultstable'. The table is used to save data from each race to a data file
  Resultstable = new Table();               //Name the new table
  Resultstable.addColumn("Race");           //Add a column named Race for Race Number
  Resultstable.addColumn("L1_Racer");       //Add a column for Lane 1 Racer ID number
  Resultstable.addColumn("L1_DQflag");      //Add a column for Lane 1 Disqualification flag
  Resultstable.addColumn("L1_React");       //Add a column for Lane 1 Reaction Time
  Resultstable.addColumn("L1_6Ft");         //Add a column for Lane 1 6 Ft time
  Resultstable.addColumn("L1_66Ft");        //Add a column for Lane 1 66 Ft time
  Resultstable.addColumn("L1_Trap");        //Add a column for Lane 1 Trap Time
  Resultstable.addColumn("L1_Speed");       //Add a column for Lane 1 Speed
  Resultstable.addColumn("L1_Elap");        //Add a column for Lane 1 Elapsed Time
  Resultstable.addColumn("L1_Ovl");         //Add a column for Lane 1 Overall race Time
  Resultstable.addColumn("L1_Winflag");     //Add a column for Lane 1 Win flag
  Resultstable.addColumn("L2_Racer");       //Add a column for Lane 2 Racer ID number
  Resultstable.addColumn("L2_DQflag");      //Add a column for Lane 2 Disqualification flag
  Resultstable.addColumn("L2_React");       //Add a column for Lane 2 Reaction Time
  Resultstable.addColumn("L2_6Ft");         //Add a column for Lane 2 6 Ft time
  Resultstable.addColumn("L2_66Ft");        //Add a column for Lane 2 66 Ft time
  Resultstable.addColumn("L2_Trap");        //Add a column for Lane 2 Trap Time
  Resultstable.addColumn("L2_Speed");       //Add a column for Lane 2 Speed
  Resultstable.addColumn("L2_Elap");        //Add a column for Lane 2 Elapsed Time
  Resultstable.addColumn("L2_Ovl");         //Add a column for Lane 2 overall race Time
  Resultstable.addColumn("L2_Winflag");     //Add a column for Lane 2 Win flag
  Resultstable.addColumn("Date");           //Add a column for race date
  Resultstable.addColumn("Time");           //Add a column for race time
  
  //Set the font
  //The font must be located in the sketch's "data" directory to load successfully
  font = loadFont("ArialNarrow-Bold-48.vlw");
  textFont(font, 50);
  
  //Draw display header and legend info
  textAlign(CENTER);                         //Set text alignment to center
  fill(255, 0, 170);                         //Set Title color
  textSize(40);                              //Set text size to 40 pixels
  text(Disp_Title,500,35);                   //Display Title
  fill(0);                                   //Set color to black
  textSize(25);                              //Set text size to 25 pixels
  text("Lane 1",250,60);                     //Display Lane 1 legend
  text("Lane 2",750,60);                     //Display Lane 2 legend
  textSize(30);                              //Set text size to 30 pixels
  text("Racer ID#",500,90);                  //Display Racer's number (ID)
  text("Reaction Time",500,132);             //Display Reaction Time legend
  text("6 Foot Time",500,174);               //Display 6 Ft Time legend
  text("66 Foot Time",500,216);              //Display 6 Ft Time legend
  text("Trap MPH",500,258);                  //Display Trap MPH legend
  text("Elapsed Time",500,300);              //Display Elapsed Time legend
  text("Overall Time",500,342);              //Display Overall Time legend
  text("Lane Status",500,384);               //Display Lane Status legend
  textAlign(LEFT);                           //Set text alignment to left
  fill(Rx, Gx, Bx);                          //Set color same as background
  rect(40,415,260,190,10);                   //Draw rectangle for Race Status box
  rect(320,415,640,190,10);                  //Draw rectangle for Race Control box
  rect(320,415,100,190,10);                  //Draw rectangle for Reset Timer box
  fill(0);                                   //Set color back to black
  text("Race Status",100,445);               //Display Race Status box legend
  text("Race Control",550,445);              //Display Race Control box legend
  textSize(25);                              //Set text size to 25 pixels
  text("Messages:",40,630);                  //Display message box legend
  fill(255, 0, 0);                           //Set color to Red
  ellipse(370,525,50,50);                    //Draw Reset Button
  textAlign(CENTER);                         //Set text alignment to center
  text("Timer",370,495);                     //Display Reset Button legend
  text("Reset",370,575);                     //Display Reset Button legend
  fill(0,0,255);                             //Set color to Blue
  ellipse(690,525,50,50);                    //Draw Print Ticket button
  text("Print",690,495);                     //Display Print Time Slip button legend
  text("Time Slip",690,575);                 //Display Print Time Slip button legend
  fill(0);                                   //Set color back to black
  text("Race #",860,480);                    //Display Race Box text header
  fill(100);                                 //Set rectangle color
  rect(910,485,30,80);                       //and draw Up/Down increment box
  fill(0,0,255);                             //Set color to Blue
  triangle(910, 520, 940, 520, 925, 490);    //Draw Up Arrow
  triangle(910, 530, 940, 530, 925, 560);    //Draw Down Arrow
  
  //Initialize text & message boxes by calling their respective subroutines
  Clear_Data();                              //Subroutine to initialize lane time text boxes
  Clear_Msgbox();                            //Subroutine to initialize Message box
  Draw_Racebox();                            //Subroutine to initialize Race Count box
  Set_Race_State();                          //Subroutine to initialize Race Status box
  Draw_NextRace_Button();                    //Call subroutine to draw the Next Race button
  Draw_StartRace_Button();                   //Call subroutine to draw the Start Race button
  
  //Check & Open the serial port at 9600 baud and buffer data till linefeed received
  try {                                      //Use the 'try' & 'catch' exception handling process
    i = Serial.list().length;                //Get the number of open ports
    if (i != 0) {                            //Do if at least one port is open
      if (i >= 2) {                          //Do if more than one open port
        //Create a list of found COM ports from which the user can pick
        for (j = 0; j < i; j++) {
          COMlist += char(j+'a') + " = " + Serial.list()[j];
          if (j+1 < i) COMlist += ",  ";
        }
        COMx = showInputDialog("Which COM port is correct? (a,b,..):\n"+COMlist);
        if (COMx==null || COMx.isEmpty()) {
          NoComflag = true;                         //Set the no COM detected flag
          exit();
        }
        else i = int(COMx.toLowerCase().charAt(0) - 'a') + 1;  //Convert selection
      }
      Portname = Serial.list()[i-1];               //Get the selected port name
      myPort = new Serial(this, Portname, 9600);   //Activate COM port at 9600 baud
      myPort.bufferUntil(lf);                      //Buffer until LF appears
      prompt = "COM Port Selected: " + Portname;   //Construct COM message & COM Port
      showMessageDialog(frame,prompt);             //and display COM Port message
    }
    else {
      showMessageDialog(frame,"No open/connected COM ports detected.\nFix problem & restart program");
      NoComflag = true;                            //Set the no COM detected flag
      exit();
    }
  }
  catch (Exception e) { //Error catch routine.
    showMessageDialog(frame,"COM port is not available (may\nbe in use by another program)");
    println("Error:", e);                          //Print the type of error in the IDE message box
    NoComflag = true;                              //Set the no COM detected flag
    exit();
  }
  
  if (!NoComflag) {                                           //Skip and exit program if no COM port detected
    //Prior saved race continuation select option
    Str1 = "RACE EVENT SELECT";
    prompt = "Do you wish to continue from a previously saved race event?";
    int input = showConfirmDialog(null, prompt, Str1, YES_NO_OPTION);  //Prompt: 0=yes, 1=no
    if (input == 0) {                                         //Prompt user for race event file to use
      Canceledflag = false;                                   //Set file select cancel flag to false
      selectInput("Select the race event file to use:", "RacefileSelect"); //Pop-up to select race plan file
      while (!Racefile_Selected) {                            //Wait until it is selected
        delay(1000); 
        if (Canceledflag) break;                              //True if file selection was canceled
      }
      if (Race_Filename.indexOf("Race_Results") >= 0) {       //Check if correct file type was selected
        Resultstable = loadTable(Race_Filename, "header");    //Load previously saved race event into table
        Race_Number = Races_Ran = Resultstable.getRowCount(); //Initialize race number indexes
        RaceContinueflag = true;                              //Set race continuation flag
        Draw_Racebox();                                       //Display current race number
        Index1 = Race_Number - 1;                             //Set table index pointer
        Redisplay_RaceResults();                              //Call subroutine to display results
      }
      else {
        showMessageDialog(frame,"INCORRECT FILE SELECTED\nMUST RESTART PROGRAM");
        println(Race_Filename);
        exit();
      }
    }
    if (!RaceContinueflag) Disp_Message(4);    //Call subroutine to display initial race start message
      else Disp_Message(3);                    //Call subroutine to display race continuation message
    Draw_Settings();
  }
} //End of "Setup" routine

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
*                                           MAIN ROUTINE                                               *
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void draw() {  //Note: Things aren't displayed on the screen until the draw loop completes.
  //GET & PROCESS SERIAL DATA FROM TIMER
  if (inString.indexOf("NRD") >= 0) {        //Check for receipt of 'Not Ready' state message
    Race_State = 0;                          //Set the race state to 0 (0=not ready)
    Set_Race_State();                        //and call subroutine to display the race state
    if (Race_Number > 0) {                   //Don't allow if no races have been ran yet
      NextRaceflag = false;                  //Disable Next Race button
      Draw_NextRace_Button();                //Call subroutine to update Next Race button display
    }
    Draw_StartRace_Button();                 //Call subroutine to update Start Race button display
    inString = "";                           //Clear 'Not Ready' message once processed
  }
  if (inString.indexOf("RDY") >= 0) {        //Check for receipt of 'Ready' state
    Race_State = 1;                          //Set the race state to 1 (1=ready)
    Clear_Data();                            //Call subroutine to clear data from text boxes
    Clear_Msgbox();                          //Call subroutine to initialize/clear Message box
    Racer1 = Racer2 = " ";                   //Clear Racer ID numbers
    Counter = j = 0;                         //Clear the race duration counter
    Clear_Race_Counter();                    //Call subroutine to clear count text box
    Set_Race_State();                        //Call subroutine to display the race state
    if (Race_Number > 0) {                   //Don't allow if no races have been ran yet
      NextRaceflag = false;                  //Disable Next Race button
      Draw_NextRace_Button();                //Call subroutine to update Next Race button display
    }
    inString = "";                           //Clear 'Ready' message once processed
    fill(250,175,50);                        //Set rectangle color
    rect(175,63,150,35,10);                  //Fill Lane 1 Racer's ID Number box with color
    rect(675,63,150,35,10);                  //Fill Lane 2 Racer's ID Number box with color
    textSize(25);                            //Set text size to 25 pixels
    textAlign(CENTER);                       //Set text alignment to center
    fill(255);                               //Set text color to white
    text("Click To Enter",250,90);           //Display Racer1 ID request
    text("Click To Enter",750,90);           //Display Racer2 ID request
    Disp_Message(11);                        //Call subroutine to display Racer ID entry message
  }
  if (inString.indexOf("STG") >= 0) {        //Check for receipt of 'Staged' state
    Race_State = 2;                          //Set the race state to 2 (2=staged)
    Set_Race_State();                        //and call subroutine to display the race state
    fill(100);                               //Set rectangle color
    Clear_Msgbox();                          //Call subroutine to clear Message box
    Draw_StartRace_Button();                 //Call subroutine to update Start Race button display
    inString = "";                           //Clear 'Staged' message once processed
  }
  if (inString.indexOf("CNT") >= 0) {        //Check for receipt of 'Countdown' state
    Race_State = 3;                          //Set the race state to 3 (3=countdown)
    Set_Race_State();                        //and call subroutine to display the race state
    fill(100);                               //Set rectangle color
    if (!Racer1flag) rect(175,63,150,35,10); //Clear Lane 1 Racer's ID Number box
    if (!Racer2flag) rect(675,63,150,35,10); //Clear Lane 2 Racer's ID Number box
    Racer1flag = false;                      //Clear Racer1 select flag if set
    Racer2flag = false;                      //Clear Racer2 select flag if set
    Clear_Msgbox();                          //Call subroutine to clear Message box
    Draw_StartRace_Button();                 //Call subroutine to update Start Race button display
    inString = "";                           //Clear 'Countdown' message once processed
  }
  if (inString.indexOf("RAC") >= 0) {        //Check for receipt of 'Racing' state
    Race_State = 4;                          //Set the race state to 4 (4=racing)
    Set_Race_State();                        //and call subroutine to display the race state
    Draw_StartRace_Button();                 //Call subroutine to update Start Race button display
    inString = "";                           //Clear 'Racing' message once processed
  }
  if (inString.indexOf("TRK") >= 0) {        //Check for receipt of Track Sensor error code
    i = inString.length() - 2;               //Get length of inString minus cr/lf characters
    Str1 = inString.substring(3, i);         //Extract lane sensor ID's from inString
    Msg1 = Msg1a + Str1 + Msg1b;             //Construct the error message to display
    Disp_Message(1);                         //Call subroutine to display track sensor error message
    inString = "";                           //Clear error message once processed
  }
  if (inString.indexOf("RST") >= 0) {        //Check for receipt of Remote RESET message
    Disp_Message(5);                         //Call subroutine to display Remote Reset message
    inString = "";                           //Clear message once processed
    if (NextRaceflag && (Races_Ran<=Max_Races)) { //Only allow if next race flag is TRUE & races ran <= 999
      Race_Number = Races_Ran + 1;           //Set the current race number
      Draw_Racebox();                        //Redisplay new race value
    }
  }
  if (inString.indexOf("Lane1") >= 0) {      //Check for receipt of Lane 1 Race times
    Time_String = inString;                  //and save it to a temp holding variable
    //Note: String has following format: "Lane1: x aaaa.aaa bbbb.bbb cccc.ccc dddd.ddd eeee.eee ffff.fff y"
    //  where x is the DQ flag, aaaa.aaa thru eeee.eee are the lane 1 times,
    //  ffff.fff is the lane 1 speed and y is the lane 1 win/loose status.
    Str1 = Time_String.substring(7, 8);      //Extract DQ status for lane 1
    if (Str1.equals("1")) DQ1flag = 1;       //And set the lane 1 DQ flag accordingly
      else DQ1flag = 0;
    Str1 = Time_String.substring(9, 17);     //Extract lane 1 Reaction time (aaaa.aaa)
    Value = float (Str1) - 1000.0;           //Convert it to a floating point number
    if (DQ1flag==1) Value = Value * -1.0;    //Make it negative if DQ flag is set
    ReactTime[0] = Truncate(Value);          //and trim to 3 decimal places
    Str1 = Time_String.substring(18, 26);    //Extract lane 1 6 Ft Marker time (bbbb.bbb)
    Value = float (Str1) - 1000.0;           //Convert it to a floating point number
    Trk6ftTime[0] = Truncate(Value);         //and trim to 3 decimal places
    Str1 = Time_String.substring(27, 35);    //Extract lane 1 66 Ft Marker time (cccc.ccc)
    Value = float (Str1) - 1000.0;           //Convert it to a floating point number
    Trk66ftTime[0] = Truncate(Value);        //and trim to 3 decimal places
    Str1 = Time_String.substring(36, 44);    //Extract lane 1 Trap time (dddd.ddd)
    Value = float (Str1) - 1000.0;           //Convert it to a floating point number
    TrapTime[0] = Truncate(Value);           //and trim to 3 decimal places
    Str1 = Time_String.substring(45, 53);    //Extract lane 1 Finish time (eeee.eee)
    Value = float (Str1) - 1000.0;           //Convert it to a floating point number
    ElapsedTime[0] = Truncate(Value);        //and trim to 3 decimal places
    Str1 = Time_String.substring(54, 62);    //Extract lane 1 speed (ffff.fff)
    Value = float (Str1) - 1000.0;           //Convert it to a floating point number
    Speed[0] = Truncate(Value);              //and trim to 3 decimal places
    Str1 = Time_String.substring(63, 64);    //Extract lane 1 win status (y)
    if (Str1.equals("1")) Win1flag = 1;      //And set the lane 1 Win flag accordingly
      else Win1flag = 0;
    inString = "";                           //Clear Race times string once processed
    if(ReactTime[0]<900.0 && ElapsedTime[0]<900.0) { //Check if lane 1 times are valid
      if (DQ1flag==0) {                      //If Reaction Time is positive
        Value = ReactTime[0] + ElapsedTime[0]; //calculate overall race time
        OverallTime[0] = Truncate(Value);    //and trim to 3 decimal places
      }
      else OverallTime[0] = ElapsedTime[0];  //OverallTime=ElapsedTime if DQ sensor tripped before green light
    }
    else OverallTime[0] = 999.999;           //Set overall time to default value if invalid lane times
  }
  if (inString.indexOf("Lane2") >= 0) {      //Check for receipt of Lane 2 Race times
    Time_String = inString;                  //and save it to a temp holding variable
    //Note: String has following format: "Lane2: x aaaa.aaa bbbb.bbb cccc.ccc dddd.ddd eeee.eee ffff.fff y"
    //  where x is the DQ flag, aaaa.aaa thru eeee.eee are the lane 2 times,
    //  eeee.eee is the lane 2 speed and y is the lane 2 win/loose status.
    Str1 = Time_String.substring(7, 8);      //Extract DQ status for lane 2 (x)
    if (Str1.equals("1")) DQ2flag = 1;       //And set the lane 2 DQ flag accordingly
      else DQ2flag = 0;
    Str1 = Time_String.substring(9, 17);     //Extract lane 2 Reaction time (aaaa.aaa)
    Value = float (Str1) - 1000.0;           //Convert it to a floating point number
    if (DQ2flag==1) Value = Value * -1.0;    //Make it negative if DQ flag is set
    ReactTime[1] = Truncate(Value);          //and trim to 3 decimal places
    Str1 = Time_String.substring(18, 26);    //Extract lane 2 6 Ft Marker time (bbbb.bbb)
    Value = float (Str1) - 1000.0;           //Convert it to a floating point number
    Trk6ftTime[1] = Truncate(Value);         //and trim to 3 decimal places
    Str1 = Time_String.substring(27, 35);    //Extract lane 2 66 Ft Marker time (cccc.ccc)
    Value = float (Str1) - 1000.0;           //Convert it to a floating point number
    Trk66ftTime[1] = Truncate(Value);        //and trim to 3 decimal places
    Str1 = Time_String.substring(36, 44);    //Extract lane 2 Trap time (dddd.ddd)
    Value = float (Str1) - 1000.0;           //Convert it to a floating point number
    TrapTime[1] = Truncate(Value);           //and trim to 3 decimal places
    Str1 = Time_String.substring(45, 53);    //Extract lane 2 Finish time (eeee.eee)
    Value = float (Str1) - 1000.0;           //Convert it to a floating point number
    ElapsedTime[1] = Truncate(Value);        //and trim to 3 decimal places
    Str1 = Time_String.substring(54, 62);    //Extract lane 2 speed (ffff.fff)
    Value = float (Str1) - 1000.0;           //Convert it to a floating point number
    Speed[1] = Truncate(Value);              //and trim to 3 decimal places
    Str1 = Time_String.substring(63, 64);    //Extract lane 2 win status
    if (Str1.equals("1")) Win2flag = 1;      //And set the lane 2 Win flag accordingly
      else Win2flag = 0;
    inString = "";                           //Clear Race times string once processed
    if(ReactTime[1]<900.0 && ElapsedTime[1]<900.0) { //Check if lane 2 times are valid
      if (DQ2flag==0) {
        Value = ReactTime[1] + ElapsedTime[1]; //calculate overall race time
        OverallTime[1] = Truncate(Value);    //and trim to 3 decimal places
      }
      else OverallTime[1] = ElapsedTime[1];  //OverallTime=ElapsedTime if DQ sensor tripped before green light
    }
    else OverallTime[1] = 999.999;           //Set overall time to default value if invalid lane times
  }
  if (inString.indexOf("FIN") >= 0) {        //Check for receipt of 'Finished' state
    Race_State = 5;                          //Set the race state to 5 (5=finished)
    Set_Race_State();                        //and call subroutine to display the race state
    Get_datetime();                          //Get & set the race date & time stamp values
    if (Race_Number == 0) Disp_Message(4);   //Call subroutine to display alternate message if no races ran
    else Disp_Message(3);                    //or display default display message
    NextRaceflag = true;                     //Enable Next Race button
    Draw_NextRace_Button();                  //Call subroutine to update Next Race button display
    inString = "";                           //Clear 'Finished' message once processed
    Get_Racer_Status();                      //Call subroutine to get race's status
    Display_Results();                       //Call subroutine to display the results
    if (Race_Number == 0) PracticeRaceflag = true; //Set flag if results are from a practice race
      else PracticeRaceflag = false;
    
    //Write results to the Resultstable if not a practice race
    if (Race_Number > Races_Ran) {           //Do only if a new race
      Races_Ran = Race_Number;               //Bump the Races Ran count
      Resultstable.addRow();                 //Add new row to the table
    }
    if (Race_Number > 0) {                   //Don't save if practice run (i.e. Race Number = #)
      Tablerow = Race_Number - 1;                                   //Set the table row index
      Resultstable.setInt(Tablerow, "Race", Race_Number);           //Save Race Number
      Resultstable.setString(Tablerow, "L1_Racer", Racer1);         //Save Lane1 racer ID number
      Resultstable.setInt(Tablerow, "L1_DQflag", DQ1flag);          //Save Lane1 reaction time
      Resultstable.setFloat(Tablerow, "L1_React", ReactTime[0]);    //Save Lane1 reaction time
      Resultstable.setFloat(Tablerow, "L1_6Ft", Trk6ftTime[0]);     //Save Lane1 6 Ft marker time
      Resultstable.setFloat(Tablerow, "L1_66Ft", Trk66ftTime[0]);   //Save Lane1 66 Ft marker time
      Resultstable.setFloat(Tablerow, "L1_Trap", TrapTime[0]);      //Save Lane1 trap time
      Resultstable.setFloat(Tablerow, "L1_Speed", Speed[0]);        //Save Lane1 speed
      Resultstable.setFloat(Tablerow, "L1_Elap", ElapsedTime[0]);   //Save Lane1 elapsed time
      Resultstable.setFloat(Tablerow, "L1_Ovl", OverallTime[0]);    //Save Lane1 overall race time
      Resultstable.setFloat(Tablerow, "L1_Winflag", Win1flag);      //Save Lane1 win status
      Resultstable.setString(Tablerow, "L2_Racer", Racer2);         //Save Lane2 racer ID number
      Resultstable.setInt(Tablerow, "L2_DQflag", DQ2flag);          //Save Lane2 reaction time
      Resultstable.setFloat(Tablerow, "L2_React", ReactTime[1]);    //Save Lane2 reaction time
      Resultstable.setFloat(Tablerow, "L2_6Ft", Trk6ftTime[1]);     //Save Lane2 6 Ft marker time
      Resultstable.setFloat(Tablerow, "L2_66Ft", Trk66ftTime[1]);   //Save Lane2 66 Ft marker time
      Resultstable.setFloat(Tablerow, "L2_Trap", TrapTime[1]);      //Save Lane2 trap time
      Resultstable.setFloat(Tablerow, "L2_Speed", Speed[1]);        //Save Lane2 speed
      Resultstable.setFloat(Tablerow, "L2_Elap", ElapsedTime[1]);   //Save Lane2 elapsed time
      Resultstable.setFloat(Tablerow, "L2_Ovl", OverallTime[1]);    //Save Lane2 overall race time
      Resultstable.setFloat(Tablerow, "L2_Winflag", Win2flag);      //Save Lane2 win status
      Resultstable.setString(Tablerow, "Date", TS_date);            //Save time slip date stamp
      Resultstable.setString(Tablerow, "Time", TS_time);            //Save time slip time stamp
      //Save all results to the datafile each time new results are available. Note that by default
      //the file is saved to the same folder that contains this sketch.
      saveTable(Resultstable, Datafile);
    }
    Create_Timeslip();                                              //Call subroutine to create time slip file
    if (Autoprintcopies > 0) {                                      //Print Time Slips?
      for (i=0; i<Autoprintcopies; i++) {                           //Loop for number of copies
        launch(Prt_Cmd);                                            //Launch Windows command to  print time slip
        delay(50);
      }
    }
  }
  //Race in progress counter
  if (Race_State==4) {                            //Do only if in Race in progress
    Sec = second();                               //Get second value from PC clock
    if (k != Sec) {                               //Do only if clock has updated
      Counter++;                                  //Increment the second count
      k = Sec;                                    //Save updated count
      Clear_Race_Counter();                       //Call subroutine to clear count text box
      fill(0, 150, 0);                            //Set color to dark green
      textAlign(RIGHT);                           //Right justify text box
      textSize(25);                               //Set text size to 25 pixels
      Str2 = nf(Counter-1) + "/10";               //Format the 10 Sec. race duration string
      text(Str2,290,572);                         //Display race duration count
    }
  }
} //End of "draw" loop

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
*                                            SUBROUTINES                                               *
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void serialEvent(Serial p) {                 //Serial data handling routine
  inString = p.readString();
}

void mouseClicked() {                        //Mouse Click handling routine
  //This routine is called when the mouse button is clicked. Mouse X & Y coordinates are tested to see
  //if the mouse pointer is over a button and will respond accordingly if so.
  x = mouseX;                                //Get mouse X-coordinate
  y = mouseY;                                //Get mouse Y-coordinate
  //Test for RESET button and send the reset command to the Arduino if so.
  if ((x>345) && (x<395) && (y>500) && (y<550)) {
    myPort.write("R");                       //Send the letter R
    myPort.write(lf);                        //followed by a line-feed
    NextRaceflag = true;                     //Enable Next Race button (in case it was disabled)
    Draw_NextRace_Button();                  //And update display
  }
  //Test for Next Race select button and perform necessary actions
  if ((x>545) && (x<625) && (y>495) && (y<555)) {
    if (NextRaceflag && (Races_Ran<=Max_Races)) { //Only allow if next race flag is TRUE & races ran <= 999
      if (Holdflag) {                        //Check if Holdflag already cleared
        Holdflag = false;                    //Reset the Hold flag
        myPort.write("C");                   //Send 'Continue' command to timer
        myPort.write(lf);                    //followed by a line-feed
      }
      Race_Number = Races_Ran + 1;           //Set the current race number
      Draw_Racebox();                        //Redisplay new race value
      myPort.write("R");                     //Send the Reset command to Arduino to ready the timer
      myPort.write(lf);                      //followed by a line-fee
    }
  }
  //Test for Race Start select button and perform necessary actions
  if ((x>445) && (x<525) && (y>495) && (y<555)) {
    if (Race_State==2 && Race_Number>Races_Ran) { //Do only if in 'Staged' state & Race# set for next race
      myPort.write("S");                     //Send the start command to Arduino
      myPort.write(lf);                      //followed by a line-feed
    }
    else {
      prompt = "TIMER NOT READY";            //Build the prompt string
      showMessageDialog(frame,prompt);       //Display Dialog
    }
  }
  //Test for Print Time Slip button and do if so.
  if ((x>665) && (x<715) && (y>500) && (y<550)) {
    if (PracticeRaceflag || Race_Number<=Races_Ran) {  //Only do if valid data is selected
      Create_Timeslip();                     //Call subroutine to create the time slip file
      launch(Prt_Cmd);                       //Launch Windows command to  print selected time slip
      prompt = "Time Slip Printed";          //Tell operator the time slip has printed
    }
    else prompt = "Must Select Race\nThat Has Been Run";
    showMessageDialog(frame,prompt);         //Display Dialog
  }
  //Test for Up Arrow button and increment Race Number value by one if so.
  if ((x>910) && (x<940) && (y>490) && (y<520)) {
    if (Race_Number < Races_Ran) {
      Race_Number++;                         //Increment Race number if < number of races ran
      Draw_Racebox();                        //and redisplay new value
      Index1 = Race_Number - 1;              //Set table index pointer
      Redisplay_RaceResults();               //Call subroutine to redisplay results
    }
    else {
      if (Race_Number == Races_Ran && Race_State == 1) {
        Race_Number++;                       //Increment Race number if ready to run the next race
        Draw_Racebox();                      //and redisplay new value
        Clear_Data();                        //Clear displayed data
        Clear_Msgbox();                      //Clear Message box
        if (Holdflag) {                      //Check if Holdflag already cleared
          Holdflag = false;                  //Reset the Hold flag
          myPort.write("C");                 //Send 'Continue' command to timer
          myPort.write(lf);                  //followed by a line-feed
        }
        textSize(30);                        //Set text size to 30 pixels
        textAlign(CENTER);                   //Set text alignment to center
        fill(255);                           //Set text color to white
        if (Racer1flag) {                    //Has a new Racer1 ID number been entered?
          Racer1 = Racer1a;                  //Restore new Racer1 ID number
          text(Racer1,250,93);               //Display Racer1 ID Number if one has been entered
        }
        if (Racer2flag) {                    //Has a new Racer2 ID number been entered?
          Racer2 = Racer2a;                  //Restore new Racer2 ID number
          text(Racer2a,750,93);              //Display Racer2 ID Number if one has been entered
        }
      }
    }
  }
  //Test for Down Arrow button and decrement Race Number value by one if so.
  if ((x>910) && (x<940) && (y>530) && (y<560)) {
    if (Race_Number > 1) {
      Race_Number--;                         //Decrement Race number if < 1
      Draw_Racebox();                        //and redisplay new value
      Index1 = Race_Number - 1;              //Set table index pointer
      Redisplay_RaceResults();               //Call subroutine to redisplay results
      if (!Holdflag) {                       //Check if Holdflag already set
        Holdflag = true;                     //Set the Hold flag
        myPort.write("H");                   //Send 'Hold' command to timer
        myPort.write(lf);                    //followed by a line-feed
      }
    }
  }
  //Test for Lane 1 racer ID box clicked and prompt user for Racer1 ID number if so.
  if ((x>175) && (x<325) && (y>63) && (y<98)) {
    //Allow only if in Ready or Staged state & next race selected
    if (Race_State==1 || Race_State==2 && Race_Number>Races_Ran) {
      myPort.write("H");                     //Send 'Hold' command to timer
      myPort.write(lf);                      //followed by a line-feed
      prompt = "Enter Lane 1 Racer's ID Number\n          (Up to 6 characters)";
      Racer1 = showInputDialog(prompt);      //Prompt user for Racer ID
      if (Racer1 == null) Racer1 = " ";      //Pad Racer1 ID# with a 'space' if empty
      else {                                 //Valid number entered
        if (Racer1.length()>6) {             //Check if longer than 6 characters
        Racer1 = Racer1.substring(0,6);      //And trim to a length of 6 if so
        }
        fill(100);                           //Set rectangle color
        rect(175,63,150,35,10);              //Clear Lane 1 Racer's ID Number box
        textSize(30);                        //Set text size to 30 pixels
        textAlign(CENTER);                   //Set text alignment to center
        fill(255);                           //Set text color to white
        text(Racer1,250,93);                 //Display Racer1 ID Number
        Racer1flag = true;                   //Set Racer1 selected flag
        Racer1a = Racer1;                    //Temp save new Racer1 ID number
      }
      myPort.write("C");                     //Send 'Continue' command to timer
      myPort.write(lf);                      //followed by a line-feed
    }
    else {
      Disp_Message(7);                       //Call subroutine to display state error message
    }
  }
  //Test for Lane 2 racer ID box clicked and prompt user for Racer2 ID number if so.
  if ((x>675) && (x<825) && (y>63) && (y<98)) {
    //Allow only if in Ready or Staged state & next race selected & in 2 racer mode
    if (Race_State==1 || Race_State==2 && Race_Number>Races_Ran) {
      if (!OneRacer_flag) {
        myPort.write("H");                   //Send 'Hold' command to timer
        myPort.write(lf);                    //followed by a line-feed
        prompt = "Enter Lane 2 Racer's ID Number\n          (Up to 6 characters)";
        Racer2 = showInputDialog(prompt);    //Prompt user for Racer ID
        if (Racer2 == null) Racer2 = " ";    //Pad Racer2 ID# with a 'space' if empty
        else {                               //Valid number entered
          if (Racer2.length()>6) {           //Check if longer than 6 characters
            Racer2 = Racer2.substring(0,6);  //And trim to a length of 6 if so
          }
          fill(100);                         //Set rectangle color
          rect(675,63,150,35,10);            //Clear Lane 2 Racer's ID Number box
          textSize(30);                      //Set text size to 30 pixels
          textAlign(CENTER);                 //Set text alignment to center
          fill(255);                         //Set text color to white
          text(Racer2,750,93);               //Display Racer2 ID Number
          Racer2flag = true;                 //Set Racer2 selected flag
          Racer2a = Racer2;                  //Temp save new Racer2 ID number
        }
        myPort.write("C");                   //Send 'Continue' command to timer
        myPort.write(lf);                    //followed by a line-feed
      }
      else Disp_Message(12);                 //Call subroutine to display state error message
    }
    else Disp_Message(7);                    //Call subroutine to display state error message
  }
}

void Clear_Data() {
  //Draw text boxes with curved corners in which the lane times and finish order will be displayed.
  //Note: This will also overwrite (clear) any previous data drawn inside
  fill(100);                                 //Set rectangle color
  rect(175,63,150,35,10);                    //Draw rectangle for Lane 1 Racer's ID Number
  rect(675,63,150,35,10);                    //Draw rectangle for Lane 2 Racer's ID Number
  rect(155,105,190,35,10);                   //Draw rectangle for Lane 1 Reaction Time
  rect(655,105,190,35,10);                   //Draw rectangle for Lane 2 Reaction Time
  rect(155,147,190,35,10);                   //Draw rectangle for Lane 1 6Ft Time
  rect(655,147,190,35,10);                   //Draw rectangle for Lane 2 6Ft Time
  rect(155,189,190,35,10);                   //Draw rectangle for Lane 1 66Ft Time
  rect(655,189,190,35,10);                   //Draw rectangle for Lane 2 66Ft Time
  rect(155,231,190,35,10);                   //Draw rectangle for Lane 1 Trap MPH
  rect(655,231,190,35,10);                   //Draw rectangle for Lane 2 Trap MPH
  rect(155,273,190,35,10);                   //Draw rectangle for Lane 1 Elapsed time
  rect(655,273,190,35,10);                   //Draw rectangle for Lane 2 Elapsed time
  rect(155,315,190,35,10);                   //Draw rectangle for Lane 1 Total time
  rect(655,315,190,35,10);                   //Draw rectangle for Lane 2 Total time
  rect(155,357,190,35,10);                   //Draw rectangle for Lane 1 race status
  rect(655,357,190,35,10);                   //Draw rectangle for Lane 2 race status
}

void Draw_Racebox() {
  //Draw the Race Count box and current Race number
  //Note: This will also overwrite (clear) any previous data drawn inside the box
  fill(100);                                 //Set rectangle color
  rect(780,485,130,80);                      //and draw it
  textSize(65);                              //Set text size to 60 pixels
  textAlign(CENTER);                         //Set text alignment to center
  fill(255);                                 //Set text color to white
  if (Race_Number > 0) text(Race_Number,845,550);
    else text("#",845,550);
}

void Set_Race_State() {
  textAlign(LEFT);                           //Set text alignment to left
  textSize(25);                              //Set text size to 25 pixels
  fill(200);                                 //Set color to dim circle & text
  ellipse(80,460,20,20);                     //Draw dimmed circle
  text("Track Not Ready",110,470);           //Draw dimmed text
  ellipse(80,485,20,20);                     //Draw dimmed circle
  text("Ready",110,495);                     //Draw dimmed text
  ellipse(80,510,20,20);                     //Draw dimmed circle
  text("Staged",110,520);                    //Draw dimmed text
  ellipse(80,535,20,20);                     //Draw dimmed circle
  text("Countdown",110,545);                 //Draw dimmed text
  ellipse(80,560,20,20);                     //Draw dimmed circle
  text("Racing",110,570);                    //Draw dimmed text
  ellipse(80,585,20,20);                     //Draw dimmed circle
  text("Finished",110,595);                  //Draw dimmed text
  switch (Race_State) {                      //Test the race state
    case 0:                                  //Do the following if it is 0
      fill(255,0,0);                         //Set color to red
      ellipse(80,460,20,20);                 //Draw red circle
      text("Track Not Ready",110,470);       //Draw red text
      break;
    case 1:                                  //Do the following if it is 1
      fill(245, 255, 0);                     //Set color yellow
      ellipse(80,485,20,20);                 //Draw yellow circle
      text("Ready",110,495);                 //Draw yellow text
      break;
    case 2:                                  //Do the following if it is 2
      fill(255);                             //Set color white
      ellipse(80,510,20,20);                 //Draw white circle
      text("Staged",110,520);                //Draw white text
      break;
    case 3:                                  //Do the following if it is 3
      fill(250,175,50);                      //Set color to amber
      ellipse(80,535,20,20);                 //Draw amber circle
      text("Countdown",110,545);             //Draw amber text
      break;
    case 4:                                  //Do the following if it is 4
      fill(0, 180, 0);                       //Set color to dark green
      ellipse(80,560,20,20);                 //Draw green circle
      text("Racing",110,570);                //Draw dark green text
      break;
    case 5:                                  //Do the following if it is 5
      fill(0, 0, 200);                       //Set color to blue
      ellipse(80,585,20,20);                 //Draw blue circle
      text("Finished",110,595);              //Draw blue text
      break;
    default:                                 //Do nothing if it is something else
      //do nothing
      break;
  }
}

void Display_Results() {    //Write race results to their respective text boxes on the screen
  Clear_Data();                                            //Call subroutine to clear old data
  textAlign(CENTER);                                       //Set text alignment to center
  fill(255);                                               //Set text color to white
  //Display Lane 1 values
  textSize(30);                                            //Set text size to 30 pixels
  text(Racer1,250,93);                                     //Display Racer1 ID Number
  textSize(40);                                            //Set text size to 40 pixels
  if (abs(ReactTime[0]) < 900.0) {                         //Check if valid reaction time value and display
    if (ReactTime[0]<0) fill(0xFFFF0000);                  //Set text color to red if negative
    text(ReactTime[0],250,137);                            //Display Lane 1 reaction time
    fill(255);                                             //Set text color back to white
  }
    else text("- - - - -",255,137);                        //Or display dashes
  if (Trk6ftTime[0] < 900.0) text(Trk6ftTime[0],250,179);  //Check if valid 6Ft time value and display
    else text("- - - - -",255,179);                        //Or display dashes
  if (Trk66ftTime[0] < 900.0) text(Trk66ftTime[0],250,221); //Check if valid 66Ft time value and display
    else text("- - - - -",255,221);                        //Or display dashes
  if (Speed[0] < 900.0) text(Speed[0],250,264);            //Check if valid speed value and display
    else text("- - - -",255,264);                          //Or display dashes
  if (ElapsedTime[0] < 900.0) text(ElapsedTime[0],250,305); //Check if valid elapsed time value and display
    else text("- - - - -",255,305);                        //Or display dashes
  if (OverallTime[0] < 900.0) text(OverallTime[0],250,347); //Check if valid overall time value and display
    else text("- - - - -",255,347);                        //Or display dashes
  fill(L1Color);                                           //Set Lane 1 status color
  text(L1status,255,389);                                  //Display Lane 1 status
  fill(255);                                               //Set text color to white
  //Display Lane 2 values
  textSize(30);                                            //Set text size to 30 pixels
  text(Racer2,750,93);                                     //Display Racer2 ID Number
  textSize(40);                                            //Set text size to 40 pixels
  if (abs(ReactTime[1]) < 900.0) {                         //Check if valid reaction time value and display
    if (ReactTime[1]<0) fill(0xFFFF0000);                  //Set text color to red if negative
    text(ReactTime[1],750,137);                            //Display Lane 2 reaction time
    fill(255);                                             //Set text color back to white
  }
    else text("- - - - -",755,137);                        //Or display dashes
  if (Trk6ftTime[1] < 900.0) text(Trk6ftTime[1],750,179);  //Check if valid 6 Ft time value and display
    else text("- - - - -",755,179);                        //Or display dashes
  if (Trk66ftTime[1] < 900.0) text(Trk66ftTime[1],750,221);  //Check if valid 66 Ft time value and display
    else text("- - - - -",755,221);                        //Or display dashes
  if (Speed[1] < 900.0) text(Speed[1],750,264);            //Check if valid speed value and display
    else text("- - - -",755,264);                          //Or display dashes
  if (ElapsedTime[1] < 900.0) text(ElapsedTime[1],750,305); //Check if valid elapsed time value and display
    else text("- - - - -",755,305);                        //Or display dashes
  if (OverallTime[1] < 900.0) text(OverallTime[1],750,347); //Check if valid overall time value and display
    else text("- - - - -",755,347);                        //Or display dashes
  fill(L2Color);                                           //Set Lane 2 status color
  text(L2status,755,389);                                  //Display Lane 2 status
}

void Redisplay_RaceResults() {     //Get & Redisplay race results for the current selected race
  //Read data from results table
  Racer1 = Resultstable.getString(Index1, "L1_Racer");      //Get Lane 1 racer ID number
  DQ1flag = Resultstable.getInt(Index1,"L1_DQflag");        //Get recorded DQ Status for Lane 1
  ReactTime[0] = Resultstable.getFloat(Index1, "L1_React"); //Get recorded reaction time for Lane 1
  Trk6ftTime[0] = Resultstable.getFloat(Index1, "L1_6Ft");  //Get recorded 6 Ft time for Lane 1
  Trk66ftTime[0] = Resultstable.getFloat(Index1, "L1_66Ft");  //Get recorded 66 Ft time for Lane 1
  Speed[0] = Resultstable.getFloat(Index1, "L1_Speed");     //Get recorded speed for Lane 1
  ElapsedTime[0] = Resultstable.getFloat(Index1, "L1_Elap");//Get recorded elapsed time for Lane 1
  OverallTime[0] = Resultstable.getFloat(Index1, "L1_Ovl"); //Get recorded overall race time for Lane 1
  Win1flag = Resultstable.getInt(Index1, "L1_Winflag");     //Get recorded win status for Lane 1
  Racer2 = Resultstable.getString(Index1, "L2_Racer");      //Get Lane 2 racer ID number
  DQ2flag = Resultstable.getInt(Index1,"L2_DQflag");        //Get recorded DQ Status for Lane 2
  ReactTime[1] = Resultstable.getFloat(Index1, "L2_React"); //Get recorded reaction time for Lane 2
  Trk6ftTime[1] = Resultstable.getFloat(Index1, "L2_6Ft");  //Get recorded 6 Ft time for Lane 2
  Trk66ftTime[1] = Resultstable.getFloat(Index1, "L2_66Ft");  //Get recorded 66 Ft time for Lane 2
  Speed[1] = Resultstable.getFloat(Index1, "L2_Speed");     //Get recorded speed for Lane 2
  ElapsedTime[1] = Resultstable.getFloat(Index1, "L2_Elap");//Get recorded elapsed time for Lane 2
  OverallTime[1] = Resultstable.getFloat(Index1, "L2_Ovl"); //Get recorded overall race time for Lane 2
  Win2flag = Resultstable.getInt(Index1, "L2_Winflag");     //Get recorded win status for Lane 2
  TS_date = Resultstable.getString(Index1, "Date");         //Get time slip date stamp 
  TS_time = Resultstable.getString(Index1, "Time");         //Get time slip time stamp
  Get_Racer_Status();                                       //Call subroutine to get racer's status
  Clear_Msgbox();                                           //Clear message text box
  Display_Results();                                        //Call subroutine to display the results
}

void Draw_StartRace_Button() {               //Subroutine to display the Start Race select button
  fill(200);                                 //Set rectangle color
  rect(445,495,80,60,10);                    //Draw rectangle
  textSize(25);                              //Set text size to 25 pixels
  textAlign(CENTER);                         //Set text alignment to center
  fill(150);                                 //Set text color to grey (Disabled)
  if (Race_State == 2) fill(0);              //Set text color to black (Enabled) if timer in the 'Staged' state
  text("Start",485,520);                     //Display "Start" inside text box top
  text("Race",485,550);                      //Display "Race" inside text box bottom
}

void Draw_NextRace_Button() {                //Subroutine to display the Next Race select button
  fill(200);                                 //Set rectangle color
  rect(545,495,80,60,10);                    //Draw rectangle
  textSize(25);                              //Set text size to 25 pixels
  textAlign(CENTER);                         //Set text alignment to center
  fill(150);                                 //Set text color to grey (Disabled)
  if (NextRaceflag) fill(0);                 //Set text color to black (Enabled) if NextRaceflag = TRUE
  text("Next",585,520);                      //Display "Next" inside text box top
  text("Race",585,550);                      //Display "Race" inside text box bottom
}

void Get_Racer_Status() {                    //Determine winner based on finish times & DQ Status
  if (Win1flag==1 && Win2flag==0) {          //Check if Lane 1 is the Winner
    if (DQ1flag==0 && DQ2flag==0) {          //Do if no lanes DQ'd
      L1Color = 0xFF00FF00;                  //Set text color to green
      L1status = "WINNER ";                  //Set Lane 1 status as the winner
      L2Color = 0xFFFFFF00;                  //Set text color to yellow
      L2status = " - - - ";                  //Set Lane 2 status as the looser
    }
    if (DQ1flag==0 && DQ2flag==1) {          //Do if only lane 2 DQ'd
      L1Color = 0xFF00FF00;                  //Set text color to green
      L1status = "WINNER ";                  //Set Lane 1 status as the winner
      if (OverallTime[1]<OverallTime[0]) {   //Did Lane 1 win by default?
        L1status = "WINNER*";                //Set Lane 1 status as the default winner
      }
      L2Color = 0xFFFF0000;                  //Set text color to red
      L2status = " DISQ  ";                  //Set Lane 2 status as the looser
    }
    if (DQ1flag==1) {                        //Do if Lane 1 won by default
      L1Color = 0xFF00FF00;                  //Set text color to green
      L1status = "WINNER*";                  //Set Lane 1 status as the default winner
      L2Color = 0xFFFF0000;                  //Set text color to red
      L2status = " DISQ  ";                  //Set Lane 2 status as disqualified
    }
  }
  if (Win1flag==0 && Win2flag==1) {          //Check if Lane 2 is the Winner
    if (DQ1flag==0 && DQ2flag==0) {          //Do if no lanes DQ'd
      L1Color = 0xFFFFFF00;                  //Set text color to yellow
      L1status = " - - - ";                  //Set Lane 1 status as the looser
      L2Color = 0xFF00FF00;                  //Set text color to green
      L2status = "WINNER ";                  //Set Lane 2 status as the winner
    }
    if (DQ1flag==1 && DQ2flag==0) {          //Do if only lane 1 DQ'd
      L1Color = 0xFFFF0000;                  //Set text color to red
      L1status = " DISQ  ";                  //Set Lane 1 status as Disqualified
      L2Color = 0xFF00FF00;                  //Set text color to green
      L2status = "WINNER ";                  //Set Lane 2 status as the winner
      if (OverallTime[0]<OverallTime[1]) {   //Did Lane 2 win by default?
        L2status = "WINNER*";                //Set Lane 1 status as the default winner
      }
    }
    if (DQ2flag==1) {                        //Do if Lane 2 won by default
      L1Color = 0xFFFF0000;                  //Set text color to red
      L1status = " DISQ  ";                  //Set Lane 1 status as disqualified
      L2Color = 0xFF00FF00;                  //Set text color to green
      L2status = "WINNER*";                  //Set Lane 2 status as the default winner
    }
  }
  if (Win1flag==0 && Win2flag==0) {          //Check if a Draw (Tie)
    if (DQ1flag==0 && DQ2flag==0) {          //Do if no lanes DQ'd
      L1Color = 0xFFFFFF00;                  //Set text color to yellow
      L1status = " - - - ";                  //Set Lane 1 status to a draw
      L2Color = 0xFFFFFF00;                  //Set text color to yellow
      L2status = " - - - ";                  //Set Lane 2 status to a draw
    }
    if (DQ1flag==1 && DQ2flag==0) {          //Do if only lane 1 DQ'd
      L1Color = 0xFFFF0000;                  //Set text color to red
      L1status = " DISQ  ";                  //Set Lane 1 status as disqualified
      L2Color = 0xFFFFFF00;                  //Set text color to yellow
      L2status = " - - - ";                  //Set Lane 2 status to a draw
    }
    if (DQ1flag==0 && DQ2flag==1) {          //Do if only lane 2 DQ'd
      L1Color = 0xFFFFFF00;                  //Set text color to yellow
      L1status = " - - - ";                  //Set Lane 1 status to a draw
      L2Color = 0xFFFF0000;                  //Set text color to red
      L2status = " DISQ  ";                  //Set Lane 2 status as disqualified
    }
    if (DQ1flag==1 && DQ2flag==1) {          //Do if both lanes DQ'd
      L1Color = 0xFFFF0000;                  //Set text color to red
      L1status = " DISQ  ";                  //Set Lane 1 status as disqualified
      L2Color = 0xFFFF0000;                  //Set text color to red
      L2status = " DISQ  ";                  //Set Lane 2 status as disqualified
    }
  } 
}

void Clear_Msgbox() {
  //Draw rectangle message box in which the error messages will be displayed
  //Note: This will also overwrite (clear) any previous data drawn inside
  fill(100);                                 //Set rectangle color
  rect(40,635,820,35,10);                    //Draw it
}

void Disp_Message(int Msg_Number) {
  Clear_Msgbox();                            //Call Clear Message Box subroutine
  textAlign(LEFT);                           //Set text alignment to left
  textSize(23);                              //Set text size to 23 pixels
  fill(255);                                 //Set color to white
  switch (Msg_Number) {                      //Test the message number
    case 0:                                  //Do nothing if it is 0
      //do nothing
      break;
    case 1:                                  //Do the following if it is 1
      text(Msg1,50,660);                     //Display message
      break;
    case 2:                                  //Do the following if it is 2
      text(Msg2,50,660);                     //Display message
      break;
    case 3:                                  //Do the following if it is 3
      text(Msg3,50,660);                     //Display message
      break;
    case 4:                                  //Do the following if it is 4
      text(Msg4,50,660);                     //Display message
      break;
    case 5:                                  //Do the following if it is 5 (Not yet implemented)
      text(Msg5,50,660);                     //Display message
      break;
    case 6:                                  //Do the following if it is 6 (Not yet implemented)
      text(Msg6,50,660);                     //Display message
      break;
    case 7:                                  //Do the following if it is 7
      text(Msg7,50,660);                     //Display message
      break;
    case 8:                                  //Do the following if it is 8
      text(Msg8,50,660);                     //Display message
      break;
    case 9:                                  //Do the following if it is 9
      text(Msg9,50,660);                     //Display message
      break;
    case 10:                                 //Do the following if it is 10
      text(Msg10,50,660);                    //Display message
      break;
    case 11:                                 //Do the following if it is 11
      text(Msg11,50,660);                    //Display message
      break;
    case 12:                                 //Do the following if it is 12
      text(Msg12,50,660);                    //Display message
      break;
    default:                                 //Do nothing if it is something else
      //do nothing
      break;
  }
}

float Truncate(float Value) {                //Function to truncate value to 3 decimal places
  return round(Value*1000.0f)/1000.0f;
}

String Resize_RacerID(String Strg) {         //Function to center Race ID text string
  if (Strg.length()>6) {                     //Check if longer than 6 characters
    Strg = Strg.substring(0,6);              //And trim to a length of 6 if so
  }
  switch (Strg.length()) {                   //Format string based on its length
    case 1:                                  //Do if 1 character long
      Strg = "  " + Strg+"   ";
      break;
    case 2:                                  //Do if 2 characters long
      Strg = "  " + Strg+"  ";
      break;
    case 3:                                  //Do if 3 characters long
      Strg = " " + Strg+"  ";
      break;
    case 4:                                  //Do if 4 characters long
      Strg = " " + Strg+" ";
      break;
    case 5:                                  //Do if 5 characters long
      Strg = Strg + " ";
      break;
    default:                                 //Do nothing if 6 characters long
      //do nothing
      break;
  }
  return Strg;
}

void Create_Timeslip() {                     //Create the Time Slip text file (33 Char/line)
  TS_output = createWriter(TS_file);         //Open Time Slip text file in the sketch directory
  j = (32 - TS_Header1.length())/2;          //Calculate number of spaces needed to center the header text
  Str1 = "";                                 //Clear variable
  for (i=0; i<j; i++) {                      //Add number of spaces required to center text
    Str1 = Str1 + " ";
  }
  TS_output.println(Str1+TS_Header1);        //Write header line 1 to the file
  j = (32 - TS_Header2.length())/2;          //Calculate number of spaces needed to center the header text
  Str1 = "";                                 //Clear variable
  for (i=0; i<j; i++) {                      //Add number of spaces required to center text
    Str1 = Str1 + " ";
  }
  TS_output.println(Str1+TS_Header2);        //Write header line 2 to the file
  TS_output.println("=================================");
  Str2 = "     Date: "+TS_date+"  "+TS_time; //Create the date/time string
  TS_output.println(Str2);                   //Write the date & time to the file header
  TS_output.println("            Race# "+nf(Race_Number,3)); //Write race# to the file header
  TS_output.println(" -Lane 1-              -Lane 2-");      //Write lane header to the file
  Str1 = Resize_RacerID(Racer1);             //Call function to format Racer1 ID
  Str2 = Resize_RacerID(Racer2);             //Call function to format Racer2 ID
  Str3 = "  "+Str1+"    Racer ID#    "+Str2; //Format Racer ID line
  TS_output.println(Str3);                   //Write Racer ID line to the file
  Str1 = " "+nf(ReactTime[0],3,3)+"   React Time   "+nf(ReactTime[1],3,3);
  if (DQ1flag==1 && DQ2flag==0) Str1 = nf(ReactTime[0],3,3)+"   React Time   "+nf(ReactTime[1],3,3);
  if (DQ1flag==0 && DQ2flag==1) Str1 = " "+nf(ReactTime[0],3,3)+"   React Time  "+nf(ReactTime[1],3,3);
  if (DQ1flag==1 && DQ2flag==1) Str1 = nf(ReactTime[0],3,3)+"   React Time  "+nf(ReactTime[1],3,3);
  TS_output.println(Str1);                   //Write reaction time line to the file
  Str1 = " "+nf(Trk6ftTime[0],3,3)+"   6 Ft Time    "+nf(Trk6ftTime[1],3,3);
  TS_output.println(Str1);                   //Write 6 Ft time line to the file
  Str1 = " "+nf(Trk66ftTime[0],3,3)+"   66 Ft Time   "+nf(Trk66ftTime[1],3,3);
  TS_output.println(Str1);                   //Write 66 Ft time line to the file
  Str1 = " "+nf(Speed[0],3,3)+" Trap Speed MPH "+nf(Speed[1],3,3);
  TS_output.println(Str1);                   //Write speed line to the file
  Str1 = " "+nf(ElapsedTime[0],3,3)+"  Elapsed Time  "+nf(ElapsedTime[1],3,3);
  TS_output.println(Str1);                   //Write elapsed time line to the file
  Str1 = " "+nf(OverallTime[0],3,3)+"  Overall Time  "+nf(OverallTime[1],3,3);
  TS_output.println(Str1);                   //Write race time line to the file
  Str1 = "  "+L1status+"    Status     "+L2status;
  TS_output.println(Str1);                   //Write status line to the file
  TS_output.println("=================================");
  j = (32 - TS_Footer1.length())/2;          //Calculate number of spaces needed to center the footer text
  Str1 = "";                                 //Clear variable
  for (i=0; i<j; i++) {                      //Add number of spaces required to center text
    Str1 = Str1 + " ";
  }
  TS_output.println(Str1+TS_Footer1);        //Write footer line 1 to the file
  j = (32 - TS_Footer2.length())/2;          //Calculate number of spaces needed to center the footer text
  Str1 = "";                                 //Clear variable
  for (i=0; i<j; i++) {                      //Add number of spaces required to center text
    Str1 = Str1 + " ";
  }
  TS_output.println(Str1+TS_Footer2);        //Write footer line 2 to the file
  TS_output.flush();                         //Writes the remaining data to the file
  TS_output.close();                         //Finishes & closes the file
}

void Get_datetime() {                        //Function: Get PC Date/Time for the Time Slip
  Yr = year();                               //Get year value from PC clock
  Mo = month();                              //Get month value from PC clock
  Day = day();                               //Get day value from PC clock
  Hr = hour();                               //Get hour value from PC clock
  Min = minute();                            //Get minute value from PC clock
  TS_date = nf(Mo,2)+"/"+nf(Day,2)+"/"+nf(Yr,4);
  TS_time = nf(Hr,2)+":"+nf(Min,2);
}

void RacefileSelect(File selection) {
  //if (selection == null) Racefile_Selected = false;
  if (selection == null) Canceledflag = true;
  else {
    Race_Filename = selection.getAbsolutePath();
    Racefile_Selected = true;
  }
}

void keyPressed() {                                     //Key Press interrupt routine
  if (keyCode==112) {                                   //Check if F1 key pressed to select Xmas tree mode
    if (Race_State<2 || Race_State==5) {                //Allow only if not in a staged or racing state
      myPort.write("H");                                //Send 'Hold' command to timer
      myPort.write(lf);                                 //followed by a line-feed
      Str1 = "XMAS TREE MODE SELECT";                   //Set prompt header
      prompt = "Select XMas Tree Mode";                 //Set prompt
      Object[] options = {"Std", "Pro", "Outlaw", "Cancel"};
      int input = showOptionDialog(null, prompt, Str1, DEFAULT_OPTION, INFORMATION_MESSAGE,null, options, null);
      if (input == 0) {                                 //Is Std mode selected?
          XmasTree_Mode = "M0";                         //Set 'Standard' Xmas Tree mode
          Stdtreeflag = true;                           //Set Std-tree flag
          Protreeflag = false;                          //Clear Pro-tree flag
          Outlawtreeflag = false;                       //Clear Outlaw-tree flag
      }
      if (input == 1) {                                 //Is Pro mode selected?
          XmasTree_Mode = "M1";                         //Set 'Pro' Xmas Tree mode
          Stdtreeflag = false;                          //Clear Std-tree flag
          Protreeflag = true;                           //Clear Pro-tree flag
          Outlawtreeflag = false;                       //Clear Outlaw-tree flag
      }
      if (input == 2) {                                 //Is Outlaw mode selected?
          XmasTree_Mode = "M2";                         //Set 'Pro' Xmas Tree mode
          Stdtreeflag = false;                          //Clear Std-tree flag
          Protreeflag = false;                          //Clear Pro-tree flag
          Outlawtreeflag = true;                        //Set Outlaw-tree flag
      }
      myPort.write(XmasTree_Mode);                      //Send Xmas Tree mode to Arduino
      myPort.write(lf);                                 //followed by a line-feed
      delay(10);                                        //Wait for Arduino timer to process
      myPort.write("C");                                //Send 'Continue' command to timer
      myPort.write(lf);                                 //followed by a line-feed
    }
    else {
      Disp_Message(9);                                  //Display state error message
    }
    Draw_Settings();                                    //Update Settings display
  }
  if (keyCode==113) {                                   //Check if F2 key pressed to select Auto Start mode
    if (Race_State<2 || Race_State==5) {                //Allow only if not in a staged or racing state
      myPort.write("H");                                //Send 'Hold' command to timer
      myPort.write(lf);                                 //followed by a line-feed
      Str1 = "AUTO START MODE SELECT";                  //Set prompt header
      if (Autostartflag) {                              //Is Auto start currently selected
        prompt = "Set Timer to Manual Start Mode?";     //Set prompt
        int input = showConfirmDialog(null, prompt, Str1, YES_NO_OPTION);  //Prompt: 0=yes, 1=no
        if (input == 0) {
          myPort.write("A0");                           //Send timer Manual-Start command to Arduino
          myPort.write(lf);                             //followed by a line-feed
          Autostartflag = false;                        //Clear Auto-Start flag
        }
      }
      else {
        prompt = "Set Timer to Auto Start Mode?";       //Set prompt
        int input = showConfirmDialog(null, prompt, Str1, YES_NO_OPTION);  //Prompt: 0=yes, 1=no
        if (input == 0) {
          myPort.write("A1");                           //Send timer Auto-Start command to Arduino
          myPort.write(lf);                             //followed by a line-feed
          Autostartflag = true;                         //Set Auto-Start flag
        }
      }
      delay(10);                                        //Wait for Arduino timer to process
      myPort.write("C");                                //Send 'Continue' command to timer
      myPort.write(lf);                                 //followed by a line-feed
    }
    else {
      Disp_Message(10);                                 //Display state error message
    }
    Draw_Settings();                                    //Update Settings display
  }
  if (keyCode==114) {                                   //Check if F3 key pressed to select single or dual racer
    if (Race_State<2 || Race_State==5) {                //Allow only if not in a staged or racing state
      myPort.write("H");                                //Send 'Hold' command to timer
      myPort.write(lf);                                 //followed by a line-feed
      Str1 = "SINGLE OR DUAL RACER SELECT";             //Set prompt header
      if (OneRacer_flag) {                              //Is a single racer currently selected
        prompt = "Set Timer to Dual Racer Mode?\nBoth lanes must have a racer";       //Set prompt
        int input = showConfirmDialog(null, prompt, Str1, YES_NO_OPTION);  //Prompt: 0=yes, 1=no
        if (input == 0) {
          myPort.write("P2");                           //Send dual racer command to Arduino timer
          myPort.write(lf);                             //followed by a line-feed
          OneRacer_flag = false;                        //Clear single racer flag
        }
      }
      else {
        prompt = "Set Timer to Single Racer Mode?\n    Racer must race in lane 1";     //Set prompt
        int input = showConfirmDialog(null, prompt, Str1, YES_NO_OPTION);  //Prompt: 0=yes, 1=no
        if (input == 0) {
          myPort.write("P1");                           //Send single racer command to Arduino timer
          myPort.write(lf);                             //followed by a line-feed
          OneRacer_flag = true;                         //Set single racer flag
        }
      }
      delay(10);                                        //Wait for Arduino timer to process
      myPort.write("C");                                //Send 'Continue' command to timer
      myPort.write(lf);                                 //followed by a line-feed
    }
    else {
      Disp_Message(10);                                 //Display state error message
    }
    Draw_Settings();                                    //Update Settings display
  }
  if (keyCode==116) {                                   //Check if F5 key pressed to select Xmas Tree lamp test
    if (Race_State<2 || Race_State==5) {                //Allow only if not in a staged or racing state
      myPort.write("T");                                //Send 'Test' command to timer
      myPort.write(lf);                                 //followed by a line-feed
    }
  }
}

void Draw_Settings() {
  if (Stdtreeflag) Str1 = "(F1) Tree: Std";            //Get Xmas tree mode setting
  if (Protreeflag) Str1 = "(F1) Tree: Pro";            //Get Xmas tree mode setting
  if (Outlawtreeflag) Str1 = "(F1) Tree: Outlaw";      //Get Xmas tree mode setting
  if (Autostartflag) Str2 = "(F2) Start: Auto";        //Get Manual/Auto start mode setting
    else Str2 = "(F2) Start: Manual";
  if (OneRacer_flag) Str3 = "(F3) Racers: One";        //Get 1/2 racers setting
    else Str3 = "(F3) Racers: Two";
  fill(Rx, Gx, Bx);                                    //Set background color
  noStroke();                                          //Disable rectangle outline
  rect(425,585,525,18);                                //Clear current values
  stroke(0);                                           //Enable rectangle outline
  fill(0);                                             //Set color to black
  textAlign(LEFT);                                     //Set text alignment
  textSize(15);                                        //Set text size to 10 pixels
  text(Str1,450,600);                                  //Display current Xmas Tree mode setting
  text(Str2,570,600);                                  //Display current timer Manual/Auto start setting
  text(Str3,710,600);                                  //Display current number of racers setting
  text("(F5) Lamp Test",840,600);                      //Display lamp test message
}

void Clear_Race_Counter() {
  fill(Rx, Gx, Bx);                                    //Set background color
  noStroke();                                          //Disable rectangle outline
  rect(180,545,115,30);                                //Clear race duration count display
  stroke(0);                                           //Enable rectangle outline
}
